import { Page } from "@/pages/base";
import { Management } from "@/management/main";
import { Box, Button, CircularProgress, Container, IconButton, Tooltip, Typography } from "@material-ui/core";
import React, { useEffect, useState } from "react";
import { Trans, useTranslation } from "react-i18next";
import { useAsyncState } from "@/hooks";
import api from "@/management/api";
import { Async } from "@/components/async";
import MaterialTable, { Action, Column } from "material-table";
import { default as StaticPage } from "@/data/page";
import { Delete, FileFind, Pencil, Refresh } from "mdi-material-ui";
import { encapsulate, one, OneOrMany } from "@/helpers";
import { Actions } from "@/components";
import { useSpacing } from "@/styles";
import { useHistory } from "react-router-dom";
import { Add, Edit } from "@material-ui/icons";
import { createPortal } from "react-dom";
import { EditStaticPageDialog } from "@/management/page/edit";
import { Confirm } from "@/components/confirm";
import useTheme from "@material-ui/core/styles/useTheme";

export const StaticPageManagement = () => {
    const { t } = useTranslation("management");
    const [ result, setPagesPromise ] = useAsyncState<StaticPage[]>();
    const [ selected, setSelected ] = useState<StaticPage[]>([]);
    const spacing = useSpacing(2);

    const updatePageList = () => {
        setPagesPromise(api.page.all());
    }

    useEffect(updatePageList, []);

    const EditStaticPageAction = ({ page }: { page: StaticPage }) => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handlePageCreation = async (page: StaticPage) => {
            await api.page.save(page);
            setOpen(false);
            updatePageList();
        }

        return <>
            <Tooltip title={ t("actions.edit") as any }>
                <IconButton onClick={ () => setOpen(true) }><Edit /></IconButton>
            </Tooltip>
            { open && createPortal(
                <EditStaticPageDialog open={ open } onSave={ handlePageCreation } page={ page } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const CreateStaticPageAction = () => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handlePageCreation = async (page: StaticPage) => {
            await api.page.save(page);
            setOpen(false);
            updatePageList();
        }

        return <>
            <Button variant="contained" color="primary" startIcon={ <Add /> } onClick={ () => setOpen(true) }>{ t("create") }</Button>
            { createPortal(
                <EditStaticPageDialog open={ open } onSave={ handlePageCreation } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const DeleteStaticPageAction = ({ page, children }: { page: OneOrMany<StaticPage>, children?: (action: any) => React.ReactNode }) => {
        const theme = useTheme();

        const handlePageDeletion = async () => {
            await api.page.remove(page);
            updatePageList();
        }

        const confirmation = <>
            { !Array.isArray(page)
                ? <Trans i18nKey="page.confirm.delete">
                    Czy na pewno chcesz usunąć stronę <strong>{ page.title.pl }</strong>?
                </Trans>
                : <>
                    { t("page.confirm.bulk-delete") }
                    <ul>
                        { page.map(page => <li key={ page.slug }>{ page.title.pl }</li>) }
                    </ul>
                </>
            }
        </>;

        return <Confirm
            onConfirm={ handlePageDeletion }
            content={ confirmation }
            confirm={ props =>
                <Button variant="contained" startIcon={ <Delete /> }
                        style={{
                            backgroundColor: theme.palette.error.dark,
                            color: theme.palette.error.contrastText,
                        }}
                        { ...props }>
                    { t("actions.delete") }
                </Button>
            }
        >
            { action => children ? children(action) : <Tooltip title={ t("actions.delete") as string }><IconButton onClick={ action }><Delete /></IconButton></Tooltip> }
        </Confirm>;
    }

    const PreviewStaticPageAction = ({ page }: { page: StaticPage }) => {
        const history = useHistory();
        const handlePagePreview = async () => history.push(`/${page.slug}`);

        return <Tooltip title={ t("actions.preview") as string }>
            <IconButton onClick={ handlePagePreview }><FileFind /></IconButton>
        </Tooltip>;
    }

    const columns: Column<StaticPage>[] = [
        {
            render: page => page.title.pl,
            title: t("page.field.title"),
        },
        {
            field: "slug",
            title: t("page.field.slug"),
        },
        {
            title: t("actions.label"),
            render: page => <Actions style={{ margin: "-1rem" }} spacing={ 0 }>
                <EditStaticPageAction page={ page } />
                <DeleteStaticPageAction page={ page } />
                <PreviewStaticPageAction page={ page } />
            </Actions>,
            sorting: false,
            width: 0,
            resizable: false,
            removable: false,
            searchable: false,
        },
    ];

    const PagePreview = ({ page }: { page: StaticPage }) =>
        <Box className={ spacing.vertical } p={ 3 }>
            <div>
                <Typography variant="subtitle2">Polski</Typography>
                <Typography variant="h2">{ page.title.pl }</Typography>
                <div dangerouslySetInnerHTML={{ __html: page.content.pl }} />
            </div>
            <div>
                <Typography variant="subtitle2">English</Typography>
                <Typography variant="h2">{ page.title.en }</Typography>
                <div dangerouslySetInnerHTML={{ __html: page.content.en }} />
            </div>
        </Box>

    return <Page>
        <Page.Header maxWidth="lg">
            <Management.Breadcrumbs>
                <Typography color="textPrimary">{ t("page.index.title") }</Typography>
            </Management.Breadcrumbs>
            <Page.Title>{ t("page.index.title") }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <CreateStaticPageAction />
                <Button onClick={ updatePageList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <Actions>
                <Typography variant="subtitle2">{ t("actions.bulk") }: </Typography>
                <DeleteStaticPageAction page={ selected }>
                    { action => <Button startIcon={ <Delete /> } onClick={ action }>{ t("actions.delete") }</Button> }
                </DeleteStaticPageAction>
            </Actions> }
            <Async async={ result } keepValue>{
                pages => <MaterialTable
                    title={ <div style={{ display: "flex", alignItems: "center" }}>{ t("page.index.title") } { result.isLoading && <CircularProgress size="1.5rem" style={{ marginLeft: "1rem" }}/> }</div> }
                    columns={ columns }
                    data={ pages }
                    detailPanel={ page => <PagePreview page={ page } /> }
                    onSelectionChange={ pages => setSelected(pages) }
                    options={{ selection: true }}
                />
            }</Async>
        </Container>
    </Page>
}

export default StaticPageManagement;
